ExpressionKind

Enum ExpressionKind 

pub enum ExpressionKind {
Show 45 variants Literal, Function, Lambda, Block, Let, Identifier, Call, If, IfLet, Match, Tuple, StructCall, DotAccess, Assignment, Return, Propagate, TryBlock, RecoverBlock, ImplBlock, Binary, Unary, Paren, Const, VariableDeclaration, RawGo, Loop, While, WhileLet, For, Break, Continue, Enum, Struct, TypeAlias, ModuleImport, Reference, Interface, IndexedAccess, Task, Defer, Select, Unit, Range, Cast, NoOp,
}
Dataless variant index of Expression, usable as an array index.

Variants



Literal



Function



Lambda



Block



Let



Identifier



Call



If



IfLet



Match



Tuple



StructCall



DotAccess



Assignment



Return



Propagate



TryBlock



RecoverBlock



ImplBlock



Binary



Unary



Paren



Const



VariableDeclaration



RawGo



Loop



While



WhileLet



For



Break



Continue



Enum



Struct



TypeAlias



ModuleImport



Reference



Interface



IndexedAccess



Task



Defer



Select



Unit



Range



Cast



NoOp

Implementations