Parser

Struct Parser 

pub struct Parser<'source> {
    pub errors: Vec<ParseError>,
    /* private fields */
}

Fields

errors: Vec<ParseError>

Implementations

Source

impl<'source> Parser<'source>

Source

impl<'source> Parser<'source>

pub fn parse_enum_definition( &mut self, doc: Option<String>, attributes: Vec<Attribute>, ) -> Expression

pub fn parse_struct_definition( &mut self, doc: Option<String>, attributes: Vec<Attribute>, ) -> Expression

pub fn parse_const_definition(&mut self, doc: Option<String>) -> Expression

pub fn parse_var_declaration(&mut self, doc: Option<String>) -> Expression

pub fn parse_impl_block(&mut self) -> Expression

pub fn parse_interface_definition(&mut self, doc: Option<String>) -> Expression

Source

impl<'source> Parser<'source>

pub fn parse_directive(&mut self) -> Expression

Source

impl<'source> Parser<'source>

pub fn parse_expression(&mut self) -> Expression

pub fn parse_atomic_expression(&mut self) -> Expression

pub fn parse_range( &mut self, start: Option<Box<Expression>>, span_start: Token<'source>, ) -> Expression

pub fn parse_struct_call(&mut self, expression: Expression) -> Expression

pub fn parse_index_expression(&mut self, expression: Expression) -> Expression

pub fn parse_function_call( &mut self, expression: Expression, type_args: Vec<Annotation>, ) -> Expression

pub fn parse_type_args(&mut self) -> Vec<Annotation>

pub fn parse_binary_operator(&mut self) -> BinaryOperator

pub fn parse_try(&mut self, expression: Expression) -> Expression

pub fn parse_block_expression(&mut self) -> Expression

pub fn parse_function_params(&mut self) -> Vec<Binding>

pub fn parse_lambda_params(&mut self) -> Vec<Binding>

pub fn parse_function( &mut self, doc: Option<String>, attributes: Vec<Attribute>, ) -> Expression

pub fn parse_field_access(&mut self, expression: Expression) -> Expression

pub fn collect_delimited_expressions( &mut self, open: TokenKind, close: TokenKind, ) -> (Vec<Expression>, bool)

pub fn parse_let(&mut self) -> Expression

pub fn parse_import(&mut self) -> Expression

pub fn parse_assignment(&mut self) -> Expression

pub fn parse_task(&mut self) -> Expression

pub fn parse_defer(&mut self) -> Expression

pub fn parse_try_block(&mut self) -> Expression

pub fn parse_recover_block(&mut self) -> Expression

pub fn parse_select(&mut self) -> Expression

pub fn with_control_flow_header<F, R>(&mut self, f: F) -> R
where F: FnOnce(&mut Self) -> R,

Source

impl<'source> Parser<'source>

Source

impl<'source> Parser<'source>

pub fn parse_pattern_allowing_or(&mut self) -> Pattern

pub fn parse_pattern(&mut self) -> Pattern

pub fn parse_binding(&mut self) -> Binding

pub fn parse_binding_allowing_or(&mut self) -> Binding

pub fn parse_binding_with_type(&mut self) -> Binding

pub fn can_start_pattern(&self) -> bool

Source

impl<'source> Parser<'source>

pub fn pratt_parse(&mut self, min_prec: u8) -> Expression

Parses by grouping together operations in expressions based on precedence.

  1. Parse a left-hand side expression (primary, unary, or prefix).
  2. Look for binary or postfix operators.
  3. For binary operators: If the operator’s precedence is higher than min_prec, parse the right-hand side recursively with the operator’s precedence.
  4. For postfix operators: Transform the current expression into a larger one.

The min_prec param sets the minimum precedence level for this parsing context.

pub fn include_in_larger_expression(&mut self, lhs: Expression) -> Expression

pub fn parse_range_end(&mut self) -> Expression

Source

impl<'source> Parser<'source>

pub fn new(tokens: Vec<Token<'source>>, source: &'source str) -> Parser<'source>

pub fn lex_and_parse_file(source: &str, file_id: u32) -> ParseResult

pub fn parse(self) -> ParseResult

pub fn parse_top_item(&mut self) -> Expression

pub fn parse_block_item(&mut self) -> Expression

pub fn at_eof(&self) -> bool