Structs
- Coercion
Info - Emit
Input - File
- File
Import - Interface
- Module
- Module
Info - Mutation
Info - Resolution
Info - Pre-computed resolution metadata from type checking.
- Unused
Info
Enums
- Call
Kind - What a call expression resolved to during type checking.
- Definition
- DotAccess
Kind - What a dot access resolved to during type checking.
- Native
Type Kind - What kind of native built-in type (Slice, Map, Channel, etc.) a call targets.
- Receiver
Coercion - Visibility